"""
PodmanInspect - Command ``podman inspect --type={TYPE}``
========================================================

This module parses the output of the ``podman inspect`` command.  This uses
the ``core.marshalling.unmarshal`` function to parse the JSON output from the
commands.  The data is stored as a dictionary.

"""

from insights import parser, CommandParser
from insights.core.marshalling import unmarshal
from insights.parsers import SkipException
from insights.specs import Specs
from insights.util import deprecated


class PodmanInspect(CommandParser, dict):
    """
    .. warning::
        This class is deprecated, please use
        :py:class:`insights.parsers.containers_inspect.ContainersInspect` instead.

    Parse the output of command "podman inspect --type=image" and "podman
    inspect --type=container".  The output of these two commands is formatted
    as JSON, so "json.loads" is an option to parse the output in the future.

    Raises:
        SkipException: If content is not provided
    """
    def __init__(self, *args, **kwargs):
        deprecated(
            PodmanInspect,
            "Please use the :class:`insights.parsers.containers_inspect.ContainersInspect` instead.",
            "3.2.25"
        )
        super(PodmanInspect, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        if not content:
            raise SkipException

        content = "\n".join(list(content))

        try:
            inspect_data = unmarshal(content)
            self.update(inspect_data[0])
        except:
            raise SkipException


@parser(Specs.podman_image_inspect)
class PodmanInspectImage(PodmanInspect):
    """
    Parse podman image inspect output using the PodmanInspect parser class.

    Sample input::

        [
            {
                "Id": "013125b8a088f45be8f85f88b5504f05c02463b10a6eea2b66809a262bb911ca",
                "Digest": "sha256:f9662cdd45e3db182372a4fa6bfff10e1c601cc785bac09ccae3b18f0bc429df",
                "RepoTags": [
                    "192.168.24.1:8787/rhosp15/openstack-rabbitmq:20190819.1",
                    "192.168.24.1:8787/rhosp15/openstack-rabbitmq:pcmklatest"
                ],
            ...

    Examples:
        >>> image['Id'] == '013125b8a088f45be8f85f88b5504f05c02463b10a6eea2b66809a262bb911ca'
        True
        >>> image['RepoTags'][0] == '192.168.24.1:8787/rhosp15/openstack-rabbitmq:20190819.1'
        True
    """
    pass


@parser(Specs.podman_container_inspect)
class PodmanInspectContainer(PodmanInspect):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.containers_inspect.ContainersInspect` instead.

    Parse podman container inspect output using the PodmanInspect parser class.

    Sample input::

        [
            {
                "ID": "66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda",
                "Created": "2019-08-21T10:38:34.753548542Z",
                "Path": "dumb-init",
                "State": {
                    "OciVersion": "1.0.1-dev",
                    "Status": "running",
                    "Running": true,
                    "Paused": false,
                },
            ...

    Examples:
        >>> container['ID'] == '66db151828e9beede0cdd9c17fc9bd5ebb5d125dd036f7230bc6b6433e5c0dda'
        True
        >>> container['Path'] == 'dumb-init'
        True
        >>> container.get('State').get('Paused') is False
        True
    """
    def __init__(self, *args, **kwargs):
        deprecated(
            PodmanInspectContainer,
            "Please use the :class:`insights.parsers.containers_inspect.ContainersInspect` instead.",
            "3.2.25"
        )
        super(PodmanInspectContainer, self).__init__(*args, **kwargs)



import pytest
import json
from mock.mock import Mock

from insights.core.dr import SkipComponent
from insights.core import filters
from insights.specs import Specs
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.semanage import LocalSpecs, users_count_map_selinux_user

SEMANGE_LOGIN_LIST_OUTPUT1 = """

Login Name           SELinux User         MLS/MCS Range        Service

%groupb               staff_u             s0-s0:c0.c1023       *
__default__          unconfined_u         s0-s0:c0.c1023       *
testa                staff_u              s0                   *
root                 unconfined_u         s0-s0:c0.c1023       *
"""

SEMANGE_LOGIN_LIST_OUTPUT2 = """

Login Name           SELinux User         MLS/MCS Range        Service

__default__          unconfined_u         s0-s0:c0.c1023       *
systemu              systemu              s0-s0:c0.c1023       *
root                 unconfined_u         s0-s0:c0.c1023       *
"""

RELATIVE_PATH = 'insights_commands/linux_users_count_map_selinux_user'


def setup_function(func):
    if Specs.selinux_users in filters._CACHE:
        del filters._CACHE[Specs.selinux_users]
    if Specs.selinux_users in filters.FILTERS:
        del filters.FILTERS[Specs.selinux_users]

    if func is test_linux_users_count_map_staff_u:
        filters.add_filter(Specs.selinux_users, ["staff_u"])
    if func is test_linux_users_count_map_more_selinux_users:
        filters.add_filter(Specs.selinux_users, ["staff_u", "unconfined_u"])
    if func is test_linux_users_count_map_staff_u_except:
        filters.add_filter(Specs.selinux_users, [])


def test_linux_users_count_map_staff_u():
    selinux_list = Mock()
    selinux_list.content = SEMANGE_LOGIN_LIST_OUTPUT1.splitlines()
    broker = {
        LocalSpecs.selinux_user_mapping: selinux_list
    }
    result = users_count_map_selinux_user(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    data = {'staff_u': 2}
    expected = DatasourceProvider(json.dumps(data), relative_path=RELATIVE_PATH)
    assert sorted(result.content) == sorted(expected.content)
    assert result.relative_path == expected.relative_path


def test_linux_users_count_map_more_selinux_users():
    selinux_list = Mock()
    selinux_list.content = SEMANGE_LOGIN_LIST_OUTPUT1.splitlines()
    broker = {
        LocalSpecs.selinux_user_mapping: selinux_list
    }
    result = users_count_map_selinux_user(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    data = {'staff_u': 2, "unconfined_u": 2}
    expected = DatasourceProvider(json.dumps(data), relative_path=RELATIVE_PATH)
    assert sorted(result.content) == sorted(expected.content)
    assert result.relative_path == expected.relative_path


def test_linux_users_count_map_staff_u_except():
    selinux_list = Mock()
    selinux_list.content = SEMANGE_LOGIN_LIST_OUTPUT2.splitlines()
    broker = {
        LocalSpecs.selinux_user_mapping: selinux_list
    }
    with pytest.raises(SkipComponent):
        users_count_map_selinux_user(broker)



from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.satellite_version import Satellite6Version
from insights.combiners import satellite_version
from insights.combiners.satellite_version import SatelliteVersion, CapsuleVersion
from insights.tests import context_wrap
from insights import SkipComponent
import pytest
import doctest


installed_rpms_5 = """
satellite-branding-5.5.0.22-1.el6sat.noarch                 Wed May 18 14:50:17 2016
satellite-doc-indexes-5.6.0-2.el6sat.noarch                 Wed May 18 14:47:49 2016
satellite-repo-5.6.0.3-1.el6sat.noarch                      Wed May 18 14:37:34 2016
satellite-schema-5.6.0.10-1.el6sat.noarch                   Wed May 18 14:53:03 2016
satyr-0.16-2.el6.x86_64                                     Wed May 18 14:16:08 2016
scdb-1.15.8-1.el6sat.noarch                                 Wed May 18 14:48:14 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el6.x86_64                                     Wed May 18 14:16:25 2016
"""

installed_rpms_60 = """
foreman-1.6.0.53-1.el7sat.noarch                            Wed May 18 14:16:25 2016
candlepin-0.9.23.11-1.el7.noarch                            Wed May 18 14:16:25 2016
katello-1.5.0.2-1.el7.noarch                                Wed May 18 14:16:25 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el6.x86_64                                     Wed May 18 14:16:25 2016
"""

installed_rpms_61 = """
foreman-1.7.2.53-1.el7sat.noarch                            Wed May 18 14:16:25 2016
candlepin-0.9.49.11-1.el7.noarch                            Wed May 18 14:16:25 2016
katello-2.2.0.17-1.el7.noarch                               Wed May 18 14:16:25 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el6.x86_64                                     Wed May 18 14:16:25 2016
"""

installed_rpms_62 = """
foreman-1.11.0.53-1.el7sat.noarch                           Wed May 18 14:16:25 2016
scl-utils-20120927-27.el7_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el7.x86_64                                     Wed May 18 14:16:25 2016
satellite-6.2.0.11-1.el7sat.noarch                          Wed May 18 14:16:25 2016
"""

installed_rpms_62_cap = """
scl-utils-20120927-27.el7_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el7.x86_64                                     Wed May 18 14:16:25 2016
satellite-capsule-6.2.0.11-1.el7sat.noarch                  Wed May 18 14:16:25 2016
"""

satellite_version_rb = """
COMMAND> cat /usr/share/foreman/lib/satellite/version.rb

module Satellite
  VERSION = "6.1.3"
end
"""

no_sat = """
scdb-1.15.8-1.el6sat.noarch                                 Wed May 18 14:48:14 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el6.x86_64                                     Wed May 18 14:16:25 2016
"""

installed_rpms_6110 = """
foreman-1.7.2.61-1.el7sat.noarch                            Wed May 18 14:16:25 2016
candlepin-0.9.49.16-1.el7.noarch                            Wed May 18 14:16:25 2016
katello-2.2.0.19-1.el7.noarch                               Wed May 18 14:16:25 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
"""

installed_rpms_6111 = """
foreman-1.7.2.62-1.el7sat.noarch                            Wed May 18 14:16:25 2016
candlepin-0.9.49.19-1.el7.noarch                            Wed May 18 14:16:25 2016
katello-2.2.0.19-1.el7.noarch                               Wed May 18 14:16:25 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
"""

installed_rpms_611x_confilct = """
foreman-1.7.2.61-1.el7sat.noarch                            Wed May 18 14:16:25 2016
candlepin-0.9.54.10-1.el7.noarch                            Wed May 18 14:16:25 2016
katello-2.3.0.19-1.el7.noarch                               Wed May 18 14:16:25 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
"""

BOTH_CAPSULE_AND_FOREMAN_PKGS_INSTALLED = """
scl-utils-20120927-27.el7_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el7.x86_64                                     Wed May 18 14:16:25 2016
satellite-capsule-6.9.0.11-1.el7sat.noarch                  Wed May 18 14:16:25 2016
foreman-1.24.1-1.el7sat.noarch                              Wed May 18 14:16:25 2016
"""

BOTH_CAPSULE_AND_SATELLITE_PKGS_INSTALLED = """
scl-utils-20120927-27.el7_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el7.x86_64                                     Wed May 18 14:16:25 2016
satellite-capsule-6.9.0.11-1.el7sat.noarch                  Wed May 18 14:16:25 2016
satellite-6.9.0.11-1.el7sat.noarch                          Wed May 18 14:16:25 2016
"""


def test_get_sat5_version():
    rpms = InstalledRpms(context_wrap(installed_rpms_5))
    expected = ('satellite-schema-5.6.0.10-1.el6sat',
                '5.6.0.10', '1.el6sat', 5, 6)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]


def test_get_sat61_version():
    rpms = InstalledRpms(context_wrap(installed_rpms_61))
    expected = ('6.1.7', '6.1.7', None, 6, 1)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]

    sat = Satellite6Version(context_wrap(satellite_version_rb))
    expected = ('6.1.3', '6.1.3', None, 6, 1)
    result = SatelliteVersion(rpms, sat)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]

    rpms = InstalledRpms(context_wrap(installed_rpms_6110))
    expected = ('6.1.10', '6.1.10', None, 6, 1)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]

    rpms = InstalledRpms(context_wrap(installed_rpms_6111))
    expected = ('6.1.11', '6.1.11', None, 6, 1)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]


def test_get_sat60():
    rpms = InstalledRpms(context_wrap(installed_rpms_60))
    expected = ('6.0.8', '6.0.8', None, 6, 0)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]


def test_get_sat61_version_both():
    rpms = InstalledRpms(context_wrap(installed_rpms_61))
    sat = Satellite6Version(context_wrap(satellite_version_rb))
    expected = ('6.1.3', '6.1.3', None, 6, 1)
    result = SatelliteVersion(rpms, sat)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.release == expected[2]
    assert result.version == expected[1]


def test_get_sat62_version():
    rpms = InstalledRpms(context_wrap(installed_rpms_62))
    expected = ('satellite-6.2.0.11-1.el7sat',
                '6.2.0.11', '1.el7sat', 6, 2)
    result = SatelliteVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.version == expected[1]
    assert result.release == expected[2]


def test_get_sat62_capsule_version():
    rpms = InstalledRpms(context_wrap(installed_rpms_62_cap))
    expected = ('satellite-capsule-6.2.0.11-1.el7sat',
                '6.2.0.11', '1.el7sat', 6, 2)
    result = CapsuleVersion(rpms, None)
    assert result.major == expected[-2]
    assert result.minor == expected[-1]
    assert result.full == expected[0]
    assert result.version == expected[1]
    assert result.release == expected[2]


def test_no_sat_installed():
    rpms = InstalledRpms(context_wrap(no_sat))
    with pytest.raises(SkipComponent) as sc:
        SatelliteVersion(rpms, None)
    assert "Not a Satellite machine" in str(sc)

    rpms = InstalledRpms(context_wrap(installed_rpms_611x_confilct))
    with pytest.raises(SkipComponent) as sc:
        SatelliteVersion(rpms, None)
    assert "unable to determine Satellite version" in str(sc)


def test_both_pkgs():
    rpms = InstalledRpms(context_wrap(BOTH_CAPSULE_AND_FOREMAN_PKGS_INSTALLED))
    capsule = CapsuleVersion(rpms, None)
    assert capsule is not None
    assert capsule.version == '6.9.0.11'

    both_satellite_and_capsule_rpms = InstalledRpms(context_wrap(BOTH_CAPSULE_AND_SATELLITE_PKGS_INSTALLED))
    satellite = SatelliteVersion(both_satellite_and_capsule_rpms, None)
    assert satellite is not None
    assert satellite.version == '6.9.0.11'

    with pytest.raises(SkipComponent):
        CapsuleVersion(both_satellite_and_capsule_rpms, satellite)


def test_doc_examples():
    sat_rpms = InstalledRpms(context_wrap(installed_rpms_62))
    cap_rpms = InstalledRpms(context_wrap(installed_rpms_62_cap))
    env = {
            'sat_ver': SatelliteVersion(sat_rpms, None),
            'cap_ver': CapsuleVersion(cap_rpms, None),
          }
    failed, total = doctest.testmod(satellite_version, globs=env)
    assert failed == 0



from insights.parsers.limits_conf import LimitsConf
from insights.tests import context_wrap

LIMITS_CONF = """
#oracle soft nproc 2047
#oracle hard nproc 16384
oracle soft nofile 1024
oracle hard nofile 65536
oracle soft stack 10240
oracle hard stack 3276
root       soft    nproc     unlimited
""".strip()

LIMITS_CONF_PATH = "/etc/security/limits.conf"

BAD_LIMITS_CONF = """

oracle
oracle soft
oracle soft nofile
root       soft    nproc     unlimitied
"""

FULL_OPTS_LIMITS_CONF = """
# ** Domain variations **
# 0: User domain
oracle      soft nofile 1024
# 1: Group domain
@dbadmins   soft nofile 1024
# 2: Wildcard domain
*           soft nofile 2048
# 3: :maxuid - exact match
:1001       soft nofile 3072
# 4: minuid: - from that number up
1000:       soft nofile 1536
# 5: minuid:maxuid - range
2000:2020   soft nofile 1600
# 6: @:maxgid - exact match
@:101       soft nofile 4096
# 7: @mingid: - from that number up
@100:       soft nofile 2560
# 8: @minuid:maxuid - range
@200:202    soft nofile 2800

# ** Type variations **
# 9: Hard type
oracle      hard nofile 8192
# 10: Both types
managers    -    nofile 10240
"""


def test_class_conf():
    # The tests handle only a single file at this point...
    ctx = context_wrap(LIMITS_CONF, path=LIMITS_CONF_PATH)
    data = LimitsConf(ctx)

    assert data.domains == \
        sorted(['oracle', 'root'])


def test_class_bad():
    # The tests handle only a single file at this point...
    ctx = context_wrap(BAD_LIMITS_CONF, path=LIMITS_CONF_PATH)
    data = LimitsConf(ctx)

    assert data.domains == []
    bad_lines = BAD_LIMITS_CONF.strip().splitlines()
    assert data.bad_lines == bad_lines


def test_class_complete():
    # The tests handle only a single file at this point...
    ctx = context_wrap(FULL_OPTS_LIMITS_CONF, path=LIMITS_CONF_PATH)
    data = LimitsConf(ctx)

    assert data.domains == \
        sorted(
            ['oracle', '@dbadmins', '*', ':1001', '1000:', '2000:2020',
            '@:101', '@100:', '@200:202', 'managers']
        )

    # Data check
    assert data.rules[0] == \
        {'domain': 'oracle', 'type': 'soft', 'item': 'nofile', 'value': 1024, 'file': LIMITS_CONF_PATH}

    # User domain match
    # oracle soft, wildcard, and oracle hard
    assert data.find_all(domain='oracle') == \
        [data.rules[x] for x in [0, 2, 9]]
    # Group domain match
    # dbadmins group and wildcard
    assert data.find_all(domain='@dbadmins') == \
        [data.rules[x] for x in [1, 2]]
    # UID domain match
    # wildcard, uid 1001 exact, and uid range 1000:
    assert data.find_all(domain=1001) == \
        [data.rules[x] for x in [2, 3, 4]]
    # UID min range domain match
    # wildcard and uid range 1000:
    assert data.find_all(domain=1002) == \
        [data.rules[x] for x in [2, 4]]
    # UID min:max range match
    # wildcard, uid range 1000:, and uid range 2000:2020
    assert data.find_all(domain=2001) == \
        [data.rules[x] for x in [2, 4, 5]]
    # GID domain match
    # wildcard, gid 101 exact, and gid range 100:
    assert data.find_all(domain='@101') == \
        [data.rules[x] for x in [2, 6, 7]]
    # GID min range domain match
    # wildcard and gid range 100:
    assert data.find_all(domain='@102') == \
        [data.rules[x] for x in [2, 7]]
    # GID min:max range match
    # wildcard, gid range 100:, and gid range 200:202
    assert data.find_all(domain='@201') == \
        [data.rules[x] for x in [2, 7, 8]]
    # soft type match
    # most rules but not the hard rule
    assert data.find_all(type='soft') == \
        [data.rules[x] for x in [0, 1, 2, 3, 4, 5, 6, 7, 8, 10]]
    # hard type match
    # hard and both
    assert data.find_all(type='hard') == \
        [data.rules[x] for x in [9, 10]]
    # No match at all
    assert data.find_all(domain='postgres', type='hard') == \
        []
    # No rules to match
    assert data.find_all() == \
        []
